#
# Author:: Noah Kantrowitz <noah@coderanger.net>
#
# Copyright 2014, Noah Kantrowitz
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import inspect

from fabric.main import find_fabfile, load_fabfile

def unwrap(task):
    """Unwrap a Fabric task to find the original function object."""
    # Unwrap
    while hasattr(task, 'wrapped'):
        task = task.wrapped
    # Smash the closure
    if task.func_code.co_name == 'inner_decorator':
        closure = dict(zip(task.func_code.co_freevars, (c.cell_contents for c in task.func_closure)))
        task = closure.get('func', closure.get('fn', task))
    return task


def visit_task(task, path):
    """Extract all needed information from a task."""
    task = unwrap(task)
    args = inspect.getargspec(task)
    return {
        'name': task.func_name,
        'path': path,
        'doc': task.__doc__,
        'cron': getattr(task, 'rundeck_cron', None),
        'argspec': {
            'args': args.args,
            'varargs': args.varargs,
            'keywords': args.keywords,
            'defaults': args.defaults,
        },
    }


def visit(c, path=()):
    """Recursively process all tasks in a fabfile."""
    ret = []
    for key, value in sorted(c.iteritems()):
        if isinstance(value, dict):
            ret.extend(visit(value, path + (key,)))
        else:
            ret.append(visit_task(value, path))
    return ret


def visit_fabfile(path=None):
    """Load and process a fabfile from the current working directory."""
    path = path or find_fabfile()
    if not path:
        raise ValueError('No fabfile detected')
    callables = load_fabfile(path)[1]
    return visit(callables)



#
# Author:: Noah Kantrowitz <noah@coderanger.net>
#
# Copyright 2014, Noah Kantrowitz
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from fabric.api import task, roles
import pytest

from fabric_rundeck import cron, hourly, daily, monthly, visitor


@pytest.fixture
def fn():
    def fn():
        pass
    return fn


class TestCron(object):
    def test_cron_task(self, fn):
        t = cron('* * * * *')(task(fn))
        data = visitor.visit_task(t, ())
        assert data['cron'] == '* * * * *'

    def test_task_cron(self, fn):
        t = task(cron('* * * * *')(fn))
        data = visitor.visit_task(t, ())
        assert data['cron'] == '* * * * *'

    def test_cron_task_hash(self, fn):
        t = cron(time={'minute': '*'})(task(fn))
        data = visitor.visit_task(t, ())
        assert data['cron'] == {'time': {'minute': '*'}}

    def test_task_cron_hash(self, fn):
        t = task(cron(time={'minute': '*'})(fn))
        data = visitor.visit_task(t, ())
        assert data['cron'] == {'time': {'minute': '*'}}


class TestHourly(object):
    def test_simple(self, fn):
        t = hourly(task(fn))
        data = visitor.visit_task(t, ())
        assert data['cron'] == '0 * * * *'


class TestDaily(object):
    def test_simple(self, fn):
        t = daily(task(fn))
        data = visitor.visit_task(t, ())
        assert data['cron'] == '0 0 * * *'


class TestMonthly(object):
    def test_simple(self, fn):
        t = monthly(task(fn))
        data = visitor.visit_task(t, ())
        assert data['cron'] == '0 0 0 * *'



#
# Author:: Noah Kantrowitz <noah@coderanger.net>
#
# Copyright 2014, Noah Kantrowitz
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os

from fabric.api import task, roles
import pytest

from fabric_rundeck import visitor


def fixture_path(*path):
    return os.path.join(os.path.dirname(__file__), 'data', *path)


class TestUnwrap(object):
    @pytest.fixture
    def fn(self):
        def fn():
            pass
        return fn

    def test_fn(self, fn):
        assert visitor.unwrap(fn) is fn

    def test_task(self, fn):
        t = task(fn)
        assert visitor.unwrap(t) is fn

    def test_taskcall(self, fn):
        t = task()(fn)
        assert visitor.unwrap(t) is fn

    def test_task_roles(self, fn):
        t = task(roles('foo')(fn))
        assert visitor.unwrap(t) is fn

    def test_taskcall_roles(self, fn):
        t = task()(roles('foo')(fn))
        assert visitor.unwrap(t) is fn

    def test_roles_task(self, fn):
        t = roles('foo')(task(fn))
        assert visitor.unwrap(t) is fn

    def test_roles_taskcall(self, fn):
        t = roles('foo')(task()(fn))
        assert visitor.unwrap(t) is fn

    def test_lambda(self):
        fn = lambda: None
        assert visitor.unwrap(fn) is fn

    def test_lambda_task(self):
        fn = lambda: None
        t = task(fn)
        assert visitor.unwrap(t) is fn


class TestVisitTask(object):
    def test_no_args(self):
        def fn():
            pass
        assert visitor.visit_task(fn, ()) == {
            'name': 'fn',
            'path': (),
            'doc': None,
            'cron': None,
            'argspec': {
                'args': [],
                'varargs': None,
                'keywords': None,
                'defaults': None,
            },
        }

    def test_simple_args(self):
        def fn(a, b):
            pass
        assert visitor.visit_task(fn, ()) == {
            'name': 'fn',
            'path': (),
            'doc': None,
            'cron': None,
            'argspec': {
                'args': ['a', 'b'],
                'varargs': None,
                'keywords': None,
                'defaults': None,
            },
        }

    def test_arg_defaults(self):
        def fn(a, b=1, c=None):
            pass
        assert visitor.visit_task(fn, ()) == {
            'name': 'fn',
            'path': (),
            'doc': None,
            'cron': None,
            'argspec': {
                'args': ['a', 'b', 'c'],
                'varargs': None,
                'keywords': None,
                'defaults': (1, None),
            },
        }

    def test_varargs(self):
        def fn(*args, **kwargs):
            pass
        assert visitor.visit_task(fn, ()) == {
            'name': 'fn',
            'path': (),
            'doc': None,
            'cron': None,
            'argspec': {
                'args': [],
                'varargs': 'args',
                'keywords': 'kwargs',
                'defaults': None,
            },
        }

    def test_docs(self):
        def fn(*args, **kwargs):
            """I am a teapot."""
            pass
        assert visitor.visit_task(fn, ()) == {
            'name': 'fn',
            'path': (),
            'doc': 'I am a teapot.',
            'cron': None,
            'argspec': {
                'args': [],
                'varargs': 'args',
                'keywords': 'kwargs',
                'defaults': None,
            },
        }


class TestVisit(object):
    def test_single(self):
        def fn():
            pass
        callables = {
            'fn': fn,
        }
        data = visitor.visit(callables)
        assert len(data) == 1
        assert data[0]['name'] == 'fn'

    def test_multi(self):
        def fn():
            pass
        def fn2():
            pass
        def fn3():
            pass
        callables = {
            'fn': fn,
            'fn2': fn2,
            'fn3': fn3,
        }
        data = visitor.visit(callables)
        assert len(data) == 3
        assert data[0]['name'] == 'fn'
        assert data[1]['name'] == 'fn2'
        assert data[2]['name'] == 'fn3'

    def test_nested(self):
        def fn():
            pass
        def fn2():
            pass
        def fn3():
            pass
        callables = {
            'fn': fn,
            'mod': {
                'fn2': fn2,
                'fn3': fn3,
            }
        }
        data = visitor.visit(callables)
        assert len(data) == 3
        assert data[0]['name'] == 'fn'
        assert data[0]['path'] == ()
        assert data[1]['name'] == 'fn2'
        assert data[1]['path'] == ('mod',)
        assert data[2]['name'] == 'fn3'
        assert data[2]['path'] == ('mod',)


class TestVisitFabfile(object):
    def test_one(self):
        data = visitor.visit_fabfile(fixture_path('fabfile_one.py'))
        assert len(data) == 3



#
# Author:: Noah Kantrowitz <noah@coderanger.net>
#
# Copyright 2014, Noah Kantrowitz
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from .visitor import unwrap

def cron(spec=None, **kwargs):
    """Decorator to mark a task as being on a cron schedule."""
    if not (spec or kwargs):
        raise ValueError('You must pass either a cron spec or keyword arguments')
    def decorator(fn):
        original = unwrap(fn)
        original.rundeck_cron = spec or kwargs
        return fn
    return decorator


def hourly(fn):
    return cron('0 * * * *')(fn)


def daily(fn):
    return cron('0 0 * * *')(fn)


def monthly(fn):
    return cron('0 0 0 * *')(fn)



#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
import os
import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True
    def run_tests(self):
        # import here, because outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)

setup(
    name = 'fabric_rundeck',
    version = '1.1',
    packages = find_packages(),
    author = 'Noah Kantrowitz',
    author_email = 'noah@coderanger.net',
    description = 'Integration helpers for Fabric and Rundeck.',
    long_description = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    license = 'Apache 2.0',
    keywords = '',
    url = 'http://github.com/coderanger/fabric-rundeck',
    classifiers = [
        #'Development Status :: 1 - Planning',
        #'Development Status :: 2 - Pre-Alpha',
        'Development Status :: 3 - Alpha',
        #'Development Status :: 4 - Beta',
        #'Development Status :: 5 - Production/Stable',
        #'Development Status :: 6 - Mature',
        #'Development Status :: 7 - Inactive',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
    zip_safe = False,
    install_requires = ['Fabric'],
    tests_require = ['pytest', 'pretend', 'flake8'],
    cmdclass = {'test': PyTest},
    # entry_points = {
    #     'console_scripts': [
    #         'foo = foo:main',
    #     ],
    # }
)
